var a = () => {
  var _Mathfloor = Math.floor;

  _Mathfloor(b);

  _Mathfloor(b);

  c: () => {
    _Mathfloor(d);

    Math.max(2, 1);
  };
};

A.b("asdas", function () {
  var _Mathmax = Math.max;
  Math.floor(d) + _Mathmax(d, e);

  _Mathmax(e, d);
});
A.b("asdas1", function () {
  var _Mathfloor2 = Math.floor;
  _Mathfloor2(d) + _Mathfloor2(d, e);
  Math.max(e, d);
});