// if statements
a.b();
foo();
a.b();
b.c();
var bar = bar1;
bar();
var baz;
var foo; // loops

function foo() {
  do {
    bar();
  } while ((bar(), false));

  for (; bar(), false;) {
    bar();
  }

  while (bar(), false) {
    bar();
  }
}