function lawl() {
  var a = 1;
  if (b) {
    return c;
  }

  if (a) {
    bar();
    return;
  }

  if (d) {
    if (g) {
      this['s'] = morebutts;
      return wat;
    }
    return boo;
  }

  haha();
  return butts;
}