(function (require, module) {
  var Hub = function Hub() {
    (0, _classCallCheck3.default)(this, Hub);
  };

  module.exports = Hub;
})(require, module, exports);

(function () {
  module.exports = function () {};
})();