class Test {
  foo() {
    var _Mathmax = Math.max;

    _Mathmax(a, d);

    _Mathmax(a, d);

    const c = function () {
      var _Mathfloor = Math.floor;

      _Mathmax(c, d);

      _Mathfloor(m);

      _Mathfloor(m);
    };
  }

  bar() {
    var _Mathmin = Math.min;

    _Mathmin(c, d);

    _Mathmin(c, d);
  }

}