function bar() {
  switch (foo) {
    case 'foo':
      foo();
      break;
    case foo.bar:
      wow();
      wat();
      break;
    case shh:
    case wow:
      baa();
      break;
    default:
      meh();
  }
}