// FIXME: The test name states that the property names should be transformed,
// but the `12e34` one isn’t transformed in the test

// prettier-ignore
({
  "ಠ_ಠ": "bar",
  "12e34": "wut",
  "\u01FC": "AE"
})
