var a = () => {
  Math.floor(b);
  Math.floor(b);
  c: () => {
    Math.floor(d);
    Math.max(2, 1);
  };
};
A.b("asdas", function() {
  Math.floor(d) + Math.max(d, e);
  Math.max(e, d);
});
A.b("asdas1", function() {
  Math.floor(d) + Math.floor(d, e);
  Math.max(e, d);
});
