function bar() {
  switch (foo) {
    case 'foo':
      return 1;
    case foo.bar:
    case wow:
      wow();
      return 3;
    case boo:
      return 4;
    case baz:
    case wat:
      return 5;
    default:
      return 0;
  }
}