(function(require, module, exports) {
  var Hub = function Hub(file, options) {
    (0, _classCallCheck3.default)(this, Hub);
  };
  module.exports = Hub;
})(require, module, exports);

(function() {
  var x = function foo() {};
  module.exports = x;
})();
