function b() {
  var _Mathfloor = Math.floor;

  _Mathfloor(as, bb);

  function d() {
    _Mathfloor(as, bb);
  }
}

const a = {
  c: () => Math.floor(bbb) + Math.floor(bbb),
  d: () => {
    var _Mathabs = Math.abs;
    var _Mathfloor2 = Math.floor;

    _Mathabs(aa);

    _Mathabs(aa);

    _Mathfloor2(aa);

    return () => {
      _Mathfloor2(aa);
    };
  },
  e: () => Math.abs(aa) + Math.abs(aa)
};

class A {
  constructor() {
    var _Mathfloor3 = Math.floor;

    let a = _Mathfloor3(b, c) + _Mathfloor3(b, c);
  }

  c() {
    var _Mathfloor4 = Math.floor;

    _Mathfloor4(asdas);

    _Mathfloor4(dasda);
  }

  d() {
    var _Mathfloor5 = Math.floor;
    var a = _Mathfloor5;
    a(aa, bb);

    _Mathfloor5(aa, bb);
  }

}

new A();