function b() {
  Math.floor(as, bb);
  function d() {
    Math.floor(as, bb);
  }
}
const a = {
  c: () => Math.floor(bbb) + Math.floor(bbb),
  d: () => {
    Math.abs(aa);
    Math.abs(aa);
    Math.floor(aa);
    return () => {
      Math.floor(aa);
    };
  },
  e: () => Math.abs(aa) + Math.abs(aa)
};
class A {
  constructor() {
    let a = Math.floor(b, c) + Math.floor(b, c);
  }
  c() {
    Math.floor(asdas);
    Math.floor(dasda);
  }
  d() {
    var a = Math.floor;
    a(aa, bb);
    Math.floor(aa, bb);
  }
}
new A();
