function a() {
  Math.max(c, a);

  const b = () => {
    var _Mathmin = Math.min;
    var _Mathfloor = Math.floor;

    const a = _Mathfloor(c);

    _Mathmin(b, a) * _Mathfloor(b);

    function c() {
      _Mathfloor(c) + _Mathmin(b, a);
    }
  };
}