class Test {
  foo() {
    Math.max(a, d);
    Math.max(a, d);
    const c = function() {
      Math.max(c, d);
      Math.floor(m);
      Math.floor(m);
    };
  }
  bar() {
    Math.min(c, d);
    Math.min(c, d);
  }
}
